setup() {
    sudo cp /usr/share/doc/audit/rules/10-base-config.rules /etc/audit/rules.d/audit.rules
    sudo augenrules --load > /dev/null
    sudo systemctl restart auditd 2> /dev/null
    sudo systemctl restart docker
    clear
}
version() {
    # Podman inside a container
    read -p "sudo podman version"
    echo ""
    sudo podman version
    echo ""
    
    # Podman inside a container
    read -p "sudo podman info"
    echo ""
    sudo podman info
}

rootless() {
    # Rootless podman
    read -p "Podman as rootless"
    echo ""

    read -p "--> podman pull alpine"
    podman pull alpine
    echo ""
    
    read -p "--> podman run --name demo --net=host --rm alpine ls"
    podman run --net=host --name demo alpine ls
    echo ""
    
    echo "Show Non Privileged containers"
    read -p "--> podman ps -a"
    podman ps -a
    echo ""
    
    echo "Show Non Privileged images"
    read -p "--> podman images"
    podman images
    echo ""
    
    echo "Show Privileged images"
    read -p "--> sudo podman images"
    sudo podman images
    echo ""
    
    read -p "--> clear"
    clear
}

userns() {
    read -p "--> buildah unshare"
    buildah unshare
    echo ""
    
    read -p "--> clear"
    clear
    
    # Podman user namespace
    read -p "Podman User Namespace Support"
    echo ""
    
    read -p "--> sudo podman run --uidmap 0:100000:5000 -d fedora sleep 1000"
    sudo podman run --net=host --uidmap 0:100000:5000 -d fedora sleep 1000
    echo ""
    
    read -p "--> sudo podman top --latest user huser"
    sudo podman top --latest user huser
    echo ""
    
    read -p "--> ps -Alf | grep sleep"
    ps -Alf | grep sleep
    echo ""
    
    read -p "--> sudo podman run --uidmap 0:200000:5000 -d fedora sleep 1000"
    sudo podman run --net=host --uidmap 0:200000:5000 -d fedora sleep 1000
    echo ""
    
    read -p "--> sudo podman top --latest user huser"
    sudo podman top --latest user huser
    echo ""
    
    read -p "--> ps -Alf | grep sleep"
    ps -Alf | grep sleep
    echo ""
    
    read -p "--> clear"
    clear
}

forkexec() {
    # Podman Fork/Exec model
    read -p "Podman Fork/Exec Model"
    echo ""
    
    read -p "--> cat /proc/self/loginuid"
    cat /proc/self/loginuid
    echo ""
    echo ""
    
    read -p "--> sudo podman run -ti fedora bash -c \"cat /proc/self/loginuid; echo\""
    sudo podman run -ti fedora bash -c "cat /proc/self/loginuid; echo"
    echo ""
    
    read -p "--> sudo docker run -ti fedora bash -c \"cat /proc/self/loginuid; echo\""
    sudo docker run -ti fedora bash -c "cat /proc/self/loginuid; echo"
    echo ""
    
    read -p "--> sudo auditctl -w /etc/shadow"
    sudo auditctl -w /etc/shadow 2>/dev/null
    echo ""

    # Showing how podman keeps track of the person trying to wreak havoc on your system
    read -p "--> sudo podman run --privileged -v /:/host fedora touch /host/etc/shadow"
    sudo podman run --privileged -v /:/host fedora touch /host/etc/shadow
    echo ""

    read -p "--> ausearch -m path -ts recent -i | grep touch | grep --color=auto 'auid=[^ ]*'"
    sudo ausearch -m path -ts recent -i | grep touch | grep --color=auto 'auid=[^ ]*'
    echo ""

    read -p "--> sudo docker run --privileged -v /:/host fedora touch /host/etc/shadow"
    sudo docker run --privileged -v /:/host fedora touch /host/etc/shadow
    echo ""

    read -p "--> ausearch -m path -ts recent -i | grep touch | grep --color=auto 'auid=[^ ]*'"
    sudo ausearch -m path -ts recent -i | grep touch | grep --color=auto 'auid=[^ ]*'
    echo ""

    read -p "--> clear"
    clear
}

top() {
    # Podman top commands
    read -p "Podman top features"
    echo ""

    read -p "--> sudo podman run -d fedora sleep 1000"
    sudo podman run -d fedora sleep 1000
    echo ""
    
    read -p "--> sudo podman top --latest label"
    sudo podman top --latest label
    echo ""
    
    read -p "--> sudo podman top --latest seccomp"
    sudo podman top --latest seccomp
    echo ""
    
    read -p "--> sudo podman top --latest capeff"
    sudo podman top --latest capeff
    echo ""
    
    read -p "--> clear"
    clear
}

pods() {
    sudo podman kill -a
    sudo podman rm -a -f
    read -p "--> sudo podman pod list"
    sudo podman pod list
    echo ""

    read -p "--> sudo podman pod --name podtest create"
    sudo podman pod create --name podtest 
    echo ""
    
    read -p "--> sudo podman pod create --pod podtest -d fedora sleep 600"
    sudo podman create --pod podtest -d fedora sleep 600
    echo ""
    
    read -p "--> sudo podman pod create --pod podtest -d fedora sleep 600"
    sudo podman create --pod podtest -d fedora sleep 600
    echo ""
    
    read -p "--> sudo podman ps"
    sudo podman ps
    echo ""

    read -p "--> sudo podman pod start podtest"
    sudo podman pod start podtest
    echo ""

    read -p "--> sudo podman ps"
    sudo podman ps
    echo ""

    read -p "--> sudo podman pod stop podtest"
    sudo podman pod stop podtest
    echo ""

    read -p "--> sudo podman ps"
    sudo podman ps
    echo ""

    read -p "--> sudo podman pod rm --force podtest"
    sudo podman pod rm --force podtest
    echo ""

    read -p "--> sudo podman pod list"
    sudo podman pod list
    echo ""
}

build() {
    echo ""
    # Already built an image with buildah installed in it
    # and made buildah the entrypoint
    read -p "--> cat Dockerfile"
    cat Dockerfile
    echo ""
    echo ""
    
    sudo mkdir -p $PWD/myvol /var/lib/mycontainer
    cat >$PWD/myvol/Dockerfile <<_EOF
FROM alpine		       
ENV foo=bar		       
LABEL colour=blue	       
_EOF
    read -p "--> sudo podman run -v \$PWD/myvol:/myvol:Z -v /var/lib/mycontainer:/var/lib/containers:Z buildah-ctr --storage-driver vfs bud -t myimage --isolation chroot /myvol"
    sudo podman run --net=host -v $PWD/myvol:/myvol:Z -v /var/lib/mycontainer:/var/lib/containers:Z buildah-ctr --storage-driver vfs bud -t myimage --isolation chroot /myvol
    echo ""

    read -p "--> sudo podman run -v /var/lib/mycontainer:/var/lib/containers:Z	buildah-ctr --storage-driver vfs images"
    sudo podman run --net=host -v /var/lib/mycontainer:/var/lib/containers:Z buildah-ctr --storage-driver vfs images
    echo ""

    read -p "--> sudo podman run -v /var/lib/mycontainer:/var/lib/containers:Z	buildah-ctr --storage-driver vfs rmi --force --all"
    sudo podman run --net=host -v /var/lib/mycontainer:/var/lib/containers:Z buildah-ctr --storage-driver vfs rmi -f --all
    echo ""    
    
    read -p "--> cleanup"
    echo "podman rm -a -f"
    sudo podman rm -a -f
    echo ""

    read -p "--> clear"
    clear
}

intro() {
    read -p "Podman Demos!"
    echo ""
    
    read -p "--> clear"
    clear
}

setup

intro

version

build

rootless

userns

forkexec

top

pods

read -p "End of Demo"
echo "Thank you!"
